var stream = require("stream");

var OutgoingMessageProto = require("http").OutgoingMessage.prototype;
if (String(OutgoingMessageProto.flush).indexOf("deprecated") !== -1) {
  // Yes, we are monkey-patching http. This method should never have been added and it was introduced on
  // the iojs fork. It was quickly deprecated and I'm 99% sure no one is actually using it.
  // See:
  // - https://github.com/marko-js/async-writer/issues/3
  // - https://github.com/nodejs/node/issues/2920
  //
  // This method causes problems since marko looks for the flush method and calls it found.
  // The `res.flush()` method is introduced by the [compression](https://www.npmjs.com/package/compression)
  // middleware, but, otherwise, it should typically not exist.
  delete OutgoingMessageProto.flush;
}

class Readable extends stream.Readable {
  constructor(template, data) {
    super();
    this._X_ = template;
    this._Y_ = data;
    this._Z_ = false;
  }

  write(data) {
    if (data != null) {
      this.push(data);
    }
  }

  end() {
    this.push(null);
  }

  _read() {
    if (this._Z_) {
      return;
    }

    this._Z_ = true;
    var template = this._X_;
    var data = this._Y_;
    var globalData = data && data.$global;
    var out = this._X_.createOut(globalData, this, undefined, template._W_);
    template.render(data, out);
    out.end();
  }
}

module.exports = function (data) {
  return new Readable(this, data);
};