var extend = require("raptor-util/extend");

function ensure(state, propertyName) {
  var proto = state.constructor.prototype;
  if (!(propertyName in proto)) {
    Object.defineProperty(proto, propertyName, {
      get: function () {
        return this.D_[propertyName];
      },
      set: function (value) {
        this.at_(propertyName, value, false /* ensure:false */);
      }
    });
  }
}

function State(component) {
  this.v_ = component;
  this.D_ = {};

  this.an_ = false;
  this.ay_ = null;
  this.ax_ = null;
  this.aR_ = null; // An object that we use to keep tracking of state properties that were forced to be dirty

  Object.seal(this);
}

State.prototype = {
  _t_: function () {
    var self = this;

    self.an_ = false;
    self.ay_ = null;
    self.ax_ = null;
    self.aR_ = null;
  },

  ar_: function (newState) {
    var state = this;
    var key;

    var rawState = this.D_;

    for (key in rawState) {
      if (!(key in newState)) {
        state.at_(key, undefined, false /* ensure:false */
        , false /* forceDirty:false */
        );
      }
    }

    for (key in newState) {
      state.at_(key, newState[key], true /* ensure:true */
      , false /* forceDirty:false */
      );
    }
  },
  at_: function (name, value, shouldEnsure, forceDirty) {
    var rawState = this.D_;

    if (shouldEnsure) {
      ensure(this, name);
    }

    if (forceDirty) {
      var forcedDirtyState = this.aR_ || (this.aR_ = {});
      forcedDirtyState[name] = true;
    } else if (rawState[name] === value) {
      return;
    }

    if (!this.an_) {
      // This is the first time we are modifying the component state
      // so introduce some properties to do some tracking of
      // changes to the state
      this.an_ = true; // Mark the component state as dirty (i.e. modified)
      this.ay_ = rawState;
      this.D_ = rawState = extend({}, rawState);
      this.ax_ = {};
      this.v_.as_();
    }

    this.ax_[name] = value;

    if (value === undefined) {
      // Don't store state properties with an undefined or null value
      delete rawState[name];
    } else {
      // Otherwise, store the new value in the component state
      rawState[name] = value;
    }
  },
  toJSON: function () {
    return this.D_;
  }
};

module.exports = State;