var EventEmitter = require("events-light");
var vdom = require("./vdom");
var VElement = vdom.bw_;
var VDocumentFragment = vdom.bx_;
var VComment = vdom.by_;
var VText = vdom.bz_;
var VComponent = vdom.bA_;
var VFragment = vdom.bB_;
var virtualizeHTML = vdom.bC_;
var RenderResult = require("../RenderResult");
var morphdom = require("./morphdom");
var attrsHelper = require("./helpers/attrs");

var EVENT_UPDATE = "update";
var EVENT_FINISH = "finish";

function State(tree) {
  this.bD_ = new EventEmitter();
  this.bE_ = tree;
  this.bF_ = false;
}

function AsyncVDOMBuilder(globalData, parentNode, parentOut) {
  if (!parentNode) {
    parentNode = new VDocumentFragment();
  }

  var state;

  if (parentOut) {
    state = parentOut.C_;
  } else {
    state = new State(parentNode);
  }

  this.bG_ = 1;
  this.bH_ = 0;
  this.bI_ = null;
  this.bJ_ = parentOut;

  this.data = {};
  this.C_ = state;
  this.ac_ = parentNode;
  this.global = globalData || {};
  this.bK_ = [parentNode];
  this.bL_ = false;
  this.bM_ = undefined;
  this.t_ = null;

  this.aa_ = null;
  this.ab_ = null;
  this.bc_ = null;
}

var proto = AsyncVDOMBuilder.prototype = {
  bq_: true,
  E_: typeof document === "object" && document,

  bc: function (component, key, ownerComponent) {
    var vComponent = new VComponent(component, key, ownerComponent);
    return this.bN_(vComponent, 0, true);
  },

  bd_: function (component, key, ownerComponent) {
    var vComponent = new VComponent(component, key, ownerComponent, true);
    this.bN_(vComponent, 0);
  },

  bN_: function (child, childCount, pushToStack) {
    this.ac_.bO_(child);
    if (pushToStack === true) {
      this.bK_.push(child);
      this.ac_ = child;
    }
    return childCount === 0 ? this : child;
  },

  element: function (tagName, attrs, key, component, childCount, flags, props) {
    var element = new VElement(tagName, attrs, key, component, childCount, flags, props);
    return this.bN_(element, childCount);
  },

  bn_: function (tagName, attrs, key, componentDef, props) {
    return this.element(tagName, attrsHelper(attrs), key, componentDef.v_, 0, 0, props);
  },

  n: function (node, component) {
    // NOTE: We do a shallow clone since we assume the node is being reused
    //       and a node can only have one parent node.
    var clone = node.__();
    this.node(clone);
    clone._N_ = component;

    return this;
  },

  node: function (node) {
    this.ac_.bO_(node);
    return this;
  },

  text: function (text, ownerComponent) {
    var type = typeof text;

    if (type != "string") {
      if (text == null) {
        return;
      } else if (type === "object") {
        if (text.toHTML) {
          return this.h(text.toHTML(), ownerComponent);
        }
      }

      text = text.toString();
    }

    this.ac_.bO_(new VText(text, ownerComponent));
    return this;
  },

  comment: function (comment, ownerComponent) {
    return this.node(new VComment(comment, ownerComponent));
  },

  html: function (html, ownerComponent) {
    if (html != null) {
      var vdomNode = virtualizeHTML(html, ownerComponent);
      this.node(vdomNode);
    }

    return this;
  },

  beginElement: function (tagName, attrs, key, component, childCount, flags, props) {
    var element = new VElement(tagName, attrs, key, component, childCount, flags, props);
    this.bN_(element, childCount, true);
    return this;
  },

  bl_: function (tagName, attrs, key, componentDef, props) {
    return this.beginElement(tagName, attrsHelper(attrs), key, componentDef.v_, 0, 0, props);
  },

  bf: function (key, component, preserve) {
    var fragment = new VFragment(key, component, preserve);
    this.bN_(fragment, null, true);
    return this;
  },

  ef: function () {
    this.endElement();
  },

  endElement: function () {
    var stack = this.bK_;
    stack.pop();
    this.ac_ = stack[stack.length - 1];
  },

  end: function () {
    this.ac_ = undefined;

    var remaining = --this.bG_;
    var parentOut = this.bJ_;

    if (remaining === 0) {
      if (parentOut) {
        parentOut.bP_();
      } else {
        this.bQ_();
      }
    } else if (remaining - this.bH_ === 0) {
      this.bR_();
    }

    return this;
  },

  bP_: function () {
    var remaining = --this.bG_;

    if (remaining === 0) {
      var parentOut = this.bJ_;
      if (parentOut) {
        parentOut.bP_();
      } else {
        this.bQ_();
      }
    } else if (remaining - this.bH_ === 0) {
      this.bR_();
    }
  },

  bQ_: function () {
    var state = this.C_;
    state.bF_ = true;
    state.bD_.emit(EVENT_FINISH, this.br_());
  },

  bR_: function () {
    var lastArray = this._last;

    var i = 0;

    function next() {
      if (i === lastArray.length) {
        return;
      }
      var lastCallback = lastArray[i++];
      lastCallback(next);

      if (!lastCallback.length) {
        next();
      }
    }

    next();
  },

  error: function (e) {
    try {
      this.emit("error", e);
    } finally {
      // If there is no listener for the error event then it will
      // throw a new Error here. In order to ensure that the async fragment
      // is still properly ended we need to put the end() in a `finally`
      // block
      this.end();
    }

    return this;
  },

  beginAsync: function (options) {
    if (this.bL_) {
      throw Error("Tried to render async while in sync mode. Note: Client side await is not currently supported in re-renders (Issue: #942).");
    }

    var state = this.C_;

    if (options) {
      if (options.last) {
        this.bH_++;
      }
    }

    this.bG_++;

    var documentFragment = this.ac_.bS_();
    var asyncOut = new AsyncVDOMBuilder(this.global, documentFragment, this);

    state.bD_.emit("beginAsync", {
      out: asyncOut,
      parentOut: this
    });

    return asyncOut;
  },

  createOut: function () {
    return new AsyncVDOMBuilder(this.global);
  },

  flush: function () {
    var events = this.C_.bD_;

    if (events.listenerCount(EVENT_UPDATE)) {
      events.emit(EVENT_UPDATE, new RenderResult(this));
    }
  },

  aj_: function () {
    return this.C_.bE_;
  },

  br_: function () {
    return this.bT_ || (this.bT_ = new RenderResult(this));
  },

  on: function (event, callback) {
    var state = this.C_;

    if (event === EVENT_FINISH && state.bF_) {
      callback(this.br_());
    } else if (event === "last") {
      this.onLast(callback);
    } else {
      state.bD_.on(event, callback);
    }

    return this;
  },

  once: function (event, callback) {
    var state = this.C_;

    if (event === EVENT_FINISH && state.bF_) {
      callback(this.br_());
    } else if (event === "last") {
      this.onLast(callback);
    } else {
      state.bD_.once(event, callback);
    }

    return this;
  },

  emit: function (type, arg) {
    var events = this.C_.bD_;
    switch (arguments.length) {
      case 1:
        events.emit(type);
        break;
      case 2:
        events.emit(type, arg);
        break;
      default:
        events.emit.apply(events, arguments);
        break;
    }
    return this;
  },

  removeListener: function () {
    var events = this.C_.bD_;
    events.removeListener.apply(events, arguments);
    return this;
  },

  sync: function () {
    this.bL_ = true;
  },

  isSync: function () {
    return this.bL_;
  },

  onLast: function (callback) {
    var lastArray = this._last;

    if (lastArray === undefined) {
      this._last = [callback];
    } else {
      lastArray.push(callback);
    }

    return this;
  },

  ai_: function (host) {
    var node = this.bM_;
    if (!node) {
      var vdomTree = this.aj_();

      if (!host) host = this.E_;
      this.bM_ = node = vdomTree.bU_(host, null);
      morphdom(node, vdomTree, host, this.t_);
    }
    return node;
  },

  toString: function (host) {
    var docFragment = this.ai_(host);
    var html = "";

    var child = docFragment.firstChild;
    while (child) {
      var nextSibling = child.nextSibling;
      if (child.nodeType != 1) {
        var container = docFragment.ownerDocument.createElement("div");
        container.appendChild(child.cloneNode());
        html += container.innerHTML;
      } else {
        html += child.outerHTML;
      }

      child = nextSibling;
    }

    return html;
  },

  then: function (fn, fnErr) {
    var out = this;
    var promise = new Promise(function (resolve, reject) {
      out.on("error", reject).on(EVENT_FINISH, function (result) {
        resolve(result);
      });
    });

    return Promise.resolve(promise).then(fn, fnErr);
  },

  catch: function (fnErr) {
    return this.then(undefined, fnErr);
  },

  isVDOM: true,

  c: function (componentDef, key, customEvents) {
    this.aa_ = componentDef;
    this.ab_ = key;
    this.bc_ = customEvents;
  }
};

proto.e = proto.element;
proto.be = proto.beginElement;
proto.ee = proto.bm_ = proto.endElement;
proto.t = proto.text;
proto.h = proto.w = proto.write = proto.html;

module.exports = AsyncVDOMBuilder;