const printJS = require("./util/printJS");

module.exports = function migrator(elNode, context) {
  const builder = context.builder;
  const attributes = elNode.attributes;
  const defAttr = attributes[0];

  if (
    !attributes.length ||
    attributes.some(
      (attr) => attr.name === "name" && attr.value.type === "Literal"
    )
  ) {
    return;
  }

  if (!defAttr || defAttr.value !== undefined) {
    context.addError(
      elNode,
      "The <macro> tag must contain a name as its first attribute, example: <macro greeting()>"
    );
    return elNode;
  }

  const name = defAttr.name;
  const params = defAttr.argument
    ? builder.parseJavaScriptParams(defAttr.argument)
    : [];
  elNode.params = [builder.identifier("macroInput")];
  elNode.argument = undefined;
  elNode.addAttribute({ name: "name", value: builder.literal(name) });
  elNode.removeAttribute(name);

  // Find usage of injected renderBody and add a renderBody param if used
  const macroWalker = context.createWalker({
    enter(child) {
      if (
        (child.type === "HtmlElement" &&
          (child.tagName === null ||
            (child.tagString && child.tagString.includes("renderBody")))) ||
        (child.type === "Identifier" && child.name === "renderBody")
      ) {
        params.push(builder.identifier("renderBody"));
        macroWalker.stop();
      }
    },
  });

  macroWalker.walk(elNode);

  params
    .slice()
    .reverse()
    .forEach((param) => {
      const name = builder.identifier(param.name);
      const value = builder.memberExpression(
        builder.identifier("macroInput"),
        name
      );
      elNode.prependChild(
        builder.scriptlet({
          value: printJS(
            builder.vars([builder.variableDeclarator(name, value)]),
            context
          ),
        })
      );
    });

  // Find all usages of the Macro and migrate the arguments to attributes.
  const templateWalker = context.createWalker({
    enter(child) {
      if (
        child.type === "HtmlElement" &&
        child.tagName === name &&
        child.argument
      ) {
        const childArgs = builder.parseJavaScriptArgs(child.argument);
        const childArgsLength = Math.min(params.length, childArgs.length);
        child.argument = undefined;

        for (let i = 0; i < childArgsLength; i++) {
          const name = params[i].name;
          const value = childArgs[i];
          child.addAttribute({ name: name, value: value });
        }
      }
    },
  });

  templateWalker.walk(context.root);
};
